#==========================================================================================================================================================================================#
# Copyright 2022 Anaplan, Inc.                                                                                                                                                             #
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without       # 
# restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the #
# Software is furnished to do so, subject to the following conditions:                                                                                                                     #
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.                                                           #
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE     #
# AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        #
# ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                                                                                    #
#==========================================================================================================================================================================================#

####################################################
## Author: Willians Santana
## Version: 2.0 ##
## Email: willians.santana@anaplan.com
#####################################################

#===============================================================================
number_of_character_ocurrences=50
header_character="-"
header_string="\n\n\n"+header_character*number_of_character_ocurrences

import requests
import json
import os
from base64 import b64encode
from OpenSSL import crypto
from requests.auth import HTTPBasicAuth
import csv
import sys
default_output_to_screen = sys.stdout
# Clear the current execution screen
os.system('clear')
#===============================================================================


######################################################################################################################################################
# This algorithm aims to populate a comma-separated value file with the Module, Line Item, List Names & IDs based on the model GUID provided
# through the variable model GUID.
# The Anaplan Transaction API Endpoints retrieve the underlying data.
# If you prefer, you can also run the Transactional API EndPoints separately from this script to obtain the same data set.
######################################################################################################################################################


######################################################################################################################################################
# Please fill in the Anaplan Public and Private Digital Certificate File Paths, If you will use the Anaplan-Certificate-Based-Authentication.
# Please, also remove the comment marks (#) of the following two lines:
# anaplan_public_certificate="Public_Digital_Certificate_File.pem"
# anaplan_private_certificate="Private_Digital_Certificate_File.pem"
# Or
# Please fill in the UserName and PassWord Variables below, If you will use the Anaplan-Username-and-Password-Authentication method
# Please, also remove the comment marks (#) of the following two lines:
######################################################################################################################################################


UserName = 'To Be Defined'
PassWord = 'To Be Defined'




# Anaplan API Authentication URL
Anaplan_API_Authentication ='https://auth.anaplan.com/token/authenticate'


# Anaplan API 2.0 Base URL
Anaplan_API_URL = 'https://api.anaplan.com/2/0/'


# Request Variable Creation
r = []


# Definition of the authentication functions: UserName&Password-based and CA-Certificate-based 

def genenerate_user_authentication(user, passw):
    auth_string = user + ':' + passw
    authorization = 'Basic ' + str(b64encode(auth_string.encode('utf-8')).decode('utf-8'))
    return authorization


def generate_ca_cert_authentication(anaplan_public_certificate,anaplan_private_certificate):

    certfile = anaplan_public_certificate
    keyfile = anaplan_private_certificate

    st_cert = open(certfile, 'rt').read()
    cert = crypto.load_certificate(crypto.FILETYPE_PEM, st_cert)

    st_key = open(keyfile, 'rt').read()
    key = crypto.load_privatekey(crypto.FILETYPE_PEM, st_key)

    pem = crypto.dump_certificate(crypto.FILETYPE_TEXT, cert)

    random_str = os.urandom(100)
    signed_str = crypto.sign(key, random_str, 'sha512')
    auth_header = 'CACertificate %s' % st_cert.replace('\n', '').replace('-----BEGIN CERTIFICATE-----', '').replace('-----END CERTIFICATE-----', '')

    encodedstr = b64encode(random_str)
    signedstr = b64encode(signed_str)

    encodedDataJson = json.dumps(encodedstr.decode('utf-8')).replace('"', '').strip()
    encodedSignedDataJson = json.dumps(signedstr.decode('utf-8')).replace('"', '').strip()

    return (auth_header, encodedDataJson, encodedSignedDataJson)


########################################################################################
# Please, choose the authentication process that best works for you:
#
# 1) CA-certificate-based authentication or 2) Username-Password-based authentication.
#
# Make sure that you comment or un-comment all the lines for each one according to your
# needs.
#
# Just one authentication method is supposed to be used per each Python Transaction API 
# script execution.
########################################################################################


########################################################################################
# Begin of CA-Certificate-based authentication code
#if __name__ == '__main__':
#    
#    authorization = generate_ca_cert_authentication(anaplan_public_certificate, anaplan_private_certificate)
#
#    encodedData = authorization[1]
#    encodedSignedData = authorization[2]
#
#    auth_header = {'Authorization': authorization[0],'Content-Type': 'application/json'}
#
#    auth_body = {'encodedData': encodedData,'encodedSignedData': encodedSignedData}
#    
#    r = requests.post(Anaplan_API_Authentication,headers=auth_header, json=auth_body)
#    
#    response = r.json()

# End of CA-Certificate-based authentication code
########################################################################################


########################################################################################
# Begin of Username-Password-based authentication code
if __name__ == '__main__':
    authorization = genenerate_user_authentication(UserName, PassWord)
    auth_header = {'Authorization': authorization,'Content-Type': 'application/json'}
    r = requests.post(Anaplan_API_Authentication, headers=auth_header)
    response = r.json()

# End of Username-Password-based authentication code
########################################################################################



tokenInfo=response['tokenInfo']
tokenValue=tokenInfo['tokenValue']


# Please, provide the modelGUID of the underlying Anaplan model
modelGUID='To Be Defined'

# Informative message indicating the start of the proces
print(header_string)
print('\n\n\n Creating comma-separated value file '+modelGUID+'.csv with the Module, Line Item, List Names & IDs based on the provided model GUID:'+modelGUID)

# The following code line defines the name of the output metadata file: <modelGUID>.csv. For example: ABCDEFGHIJKLMNOPQRSTUVWXYZ123456.csv
sys.stdout = open(modelGUID+".csv", "w",encoding="utf-8")





# myHeaders definition
myHeaders = {'Authorization':'AnaplanAuthToken '+tokenValue}






#Retrieve the module and line item metadata of a specific model

try:
    url=Anaplan_API_URL+'models/'+modelGUID+"/modules"

    r = requests.get(url, headers=myHeaders)
    response=r.json()
    moduleList=response['modules']

    print('\n\n\nModule ID,Module Name')
 

    try:
        for everysinglemodule in moduleList:

            url=Anaplan_API_URL+modelGUID+"/modules/"+everysinglemodule['id']
            r = requests.get(url, headers=myHeaders)
            response=r.json()
            print('_'+everysinglemodule['id']+'_,'+everysinglemodule['name'])
    except: pass


except: pass


try:
    
    #Module and Line Item Metadata Printing Out System
    print('\n\n\nModule ID, Module Name, Line Item ID,Line Item Name')


    url=Anaplan_API_URL+'models/'+modelGUID+"/lineItems"
    r = requests.get(url, headers=myHeaders)
    response=r.json()
    modelLineItemList=response['items']

    try:
        for lineItemDetail in modelLineItemList: print('_'+lineItemDetail['moduleId']+'_, '+lineItemDetail['moduleName']+','+'_'+lineItemDetail['id']+'_, '+lineItemDetail['name'])
    except: pass
    
except: pass



try:
    
    #Retrieve the list metadata of a specific model
    print('\n\n\nList ID, List Name')
    
    url=Anaplan_API_URL+'models/'+modelGUID+"/lists"
    r = requests.get(url, headers=myHeaders)
    response=r.json()
    modelListOfLists=response['lists']

    try:
        for everysinglelist in modelListOfLists: print('_'+everysinglelist['id']+'_,'+everysinglelist['name'])
    except: pass


except: pass


sys.stdout = default_output_to_screen

# Informative message indicating the end of the process
print(header_string)
print('\n\n\n The comma-separated value file '+modelGUID+'.csv with the Module, Line Item, List Names & IDs based on requested model GUID has been created in the same directory where you started running this Python script.')
print(header_string)

sys.stdout.close()
